# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.attributes import Attributes
from ...generated_types.types.country import Country
from ...generated_types.types.entities import Entities
from .identifier import Identifier
from .relationship_count import RelationshipCount
from .source_count_info import SourceCountInfo


class EmbeddedEntity(pydantic_v1.BaseModel):
    """
    Fields of an entity when nested within other data structures.
    """

    id: str = pydantic_v1.Field()
    """
    Unique identifier of the entity
    """

    label: str = pydantic_v1.Field()
    """
    Display name of the entity
    """

    degree: int = pydantic_v1.Field()
    """
    Number of outgoing relationships
    """

    closed: bool = pydantic_v1.Field()
    """
    True if the entity existed in the past but not at the present time, otherwise false. Always false for data curation.
    """

    entity_url: str = pydantic_v1.Field()
    """
    Convenience URL to the entity in the API.
    """

    pep: bool = pydantic_v1.Field()
    """
    True if the entity has the ["Politically Exposed Person (PEP)" risk factor](/sayari-library/ontology/risk-factors#politically-exposed-person-pep-), otherwise false.
    """

    psa_id: typing.Optional[str] = None
    psa_count: int = pydantic_v1.Field()
    """
    Number of entities that are Possibly the Same As (PSA) the entity.
    """

    sanctioned: bool = pydantic_v1.Field()
    """
    True if the entity has the ["Sanctioned" risk factor](/sayari-library/ontology/risk-factors#sanctioned), otherwise false.
    """

    type: Entities = pydantic_v1.Field()
    """
    The [entity type](/sayari-library/ontology/entities).
    """

    identifiers: typing.List[Identifier]
    countries: typing.List[Country] = pydantic_v1.Field()
    """
    Entity [country](/sayari-library/ontology/enumerated-types#country)
    """

    source_count: typing.Dict[str, SourceCountInfo] = pydantic_v1.Field()
    """
    Number of records associated with the entity, grouped by source.
    """

    addresses: typing.List[str] = pydantic_v1.Field()
    """
    List of physical addresses associated with the entity. See more [here](/sayari-library/ontology/attributes#address)
    """

    trade_count: typing.Dict[str, int]
    date_of_birth: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Birth date of a person. See more [here](/sayari-library/ontology/attributes#date-of-birth)
    """

    relationship_count: RelationshipCount
    user_relationship_count: RelationshipCount
    attribute_count: typing.Dict[Attributes, int] = pydantic_v1.Field()
    """
    Count of attributes for a given [attribute type](/sayari-library/ontology/attributes)
    """

    user_attribute_count: typing.Dict[Attributes, int] = pydantic_v1.Field()
    """
    Count of user-created attributes for a given [attribute type](/sayari-library/ontology/attributes)
    """

    related_entities_count: int
    user_related_entities_count: int
    user_record_count: int
    reference_id: typing.Optional[str] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...shared_types.types.record_details import RecordDetails
from .record_references import RecordReferences


class GetRecordResponse(RecordDetails):
    """
    OK

    Examples
    --------
    from sayari import GetRecordResponse, QualifiedCount, RecordReferences

    GetRecordResponse(
        id="74cf0fc2a62f9c8f4e88f8a0b3ffcca4%2FF0000110%2F1682970471254",
        label="Company Record from Hong Kong Companies Registry",
        source="74cf0fc2a62f9c8f4e88f8a0b3ffcca4",
        publication_date="2023-05-01",
        acquisition_date="2023-05-01",
        references_count=1,
        record_url="/v1/record/74cf0fc2a62f9c8f4e88f8a0b3ffcca4%2FF0000110%2F1682970471254",
        source_url="https://data.gov.hk/en-data/dataset/hk-cr-crdata-list-addr",
        document_urls=[
            "/document/74cf0fc2a62f9c8f4e88f8a0b3ffcca4%2FF0000110%2F1682970471254/file/json%2FF0000110.json"
        ],
        references=RecordReferences(
            next=False,
            offset=0,
            limit=100,
            size=QualifiedCount(
                count=1,
                qualifier="eq",
            ),
            data=[
                {
                    "entity": {
                        "id": "YUc8LtKFCpAbUIBGK8nQpw",
                        "label": "Bank of China Ltd.",
                        "degree": 22179,
                        "entity_url": "/v1/entity/YUc8LtKFCpAbUIBGK8nQpw",
                        "pep": false,
                        "psa_id": "81201",
                        "psa_count": 2,
                        "sanctioned": false,
                        "closed": false,
                        "translated_label": "Bank Of China Limited",
                        "company_type": "\u4e0a\u699c\u6295\u8d44\u673a\u6784",
                        "registration_date": "Registered 2015-08-18",
                        "latest_status": {"status": "active", "date": "2019-05-20"},
                        "trade_count": {"sent": 1, "received": 1806},
                        "type": "company",
                        "identifiers": [
                            {
                                "value": "54930053HGCFWVHYZX42",
                                "type": "lei",
                                "label": "Lei",
                            },
                            {
                                "value": "0001378999",
                                "type": "usa_sec_cik_number",
                                "label": "Usa Sec Cik Number",
                            },
                            {
                                "value": "03988",
                                "type": "hkg_stock_code",
                                "label": "Hkg Stock Code",
                            },
                            {
                                "value": "92751",
                                "type": "xxx_edi_global_issuer_id",
                                "label": "Xxx Edi Global Issuer Id",
                            },
                            {
                                "value": "C320201",
                                "type": "xxx_acuris_id",
                                "label": "Xxx Acuris Id",
                            },
                        ],
                        "addresses": [
                            "No. 1 Fuxingmen Nei Dajie, Beijing 100818, China",
                            "CHN",
                            "No. 1 Fuxingmen Nei Dajie, Beijing 100818, China China",
                        ],
                        "countries": [
                            "USA",
                            "JPN",
                            "CHN",
                            "FRA",
                            "PHL",
                            "HKG",
                            "KOR",
                            "NZL",
                            "GBR",
                            "LKA",
                        ],
                        "relationship_count": {
                            "receiver_of": 1806,
                            "linked_to": 68,
                            "branch_of": 161,
                            "has_branch": 11624,
                            "beneficial_owner_of": 1,
                            "has_beneficial_owner": 3,
                            "party_to": 1,
                            "has_officer": 35,
                            "has_manager": 21,
                            "shareholder_of": 289,
                            "has_shareholder": 244,
                            "notify_party_of": 2,
                            "shipper_of": 1,
                            "has_director": 187,
                            "has_member_of_the_board": 17,
                            "has_supervisor": 34,
                            "owner_of": 1443,
                            "ships_to": 1,
                            "receives_from": 114,
                            "has_subsidiary": 103,
                            "has_legal_representative": 12,
                            "issuer_of": 6350,
                        },
                        "source_count": {
                            "eb361ba33d05a9d15b2d28aea739362a": {
                                "count": 11,
                                "label": "USA SEC Central Index Key Database",
                            },
                            "2b618f1996252fe537a6d998ae14c9b2": {
                                "count": 1,
                                "label": "UK Corporate Registry Confirmation Statements",
                            },
                            "a8c6ee1cd4dfc952105ee8c0e4836f08": {
                                "count": 151,
                                "label": "Acuris Risk Intelligence KYC6 (3rd Party Data)",
                            },
                            "1fbdc8f3abdf32274f4c7657c048294a": {
                                "count": 4,
                                "label": "China CNinfo Shanghai/Shenzhen Stock Exchange Database",
                            },
                            "e458b3b7bc1392d0723f6aa5a9fe4df1": {
                                "count": 5,
                                "label": "Japan METI gBizINFO Database",
                            },
                            "fcfa3d1c6b5f9744188fc01d0999fb76": {
                                "count": 405,
                                "label": "China SAIC",
                            },
                            "bf68c3a9a482c02f1f76342feb79af8d": {
                                "count": 1,
                                "label": "China Company Directory (Xin Gongshang Minglu)(Web Crawled Data)",
                            },
                            "a025b3503797a9cd1e1964dec6943594": {
                                "count": 3054,
                                "label": "Hong Kong Stock Exchange - Shareholding Disclosures",
                            },
                            "e0a238bcfc2f81ed9e5f345c0c7068f7": {
                                "count": 6,
                                "label": "USA Department of Labor Form 5500 Filings Database",
                            },
                            "5789a1459548517db4d61679a507be1a": {
                                "count": 1549,
                                "label": "USA SEC Securities Issuers",
                            },
                            "78586b6984ecb00e81563ae3d31c9227": {
                                "count": 1,
                                "label": "Myanmar DICA",
                            },
                            "9615bab28dddcc89548c928ab192ee7c": {
                                "count": 3,
                                "label": "Sri Lanka Imports & Exports (January 2023 - Present)",
                            },
                            "be720fea19724defd6aa652dfd5c35ed": {
                                "count": 1,
                                "label": "China Company Directory (Lanjing Zhengxin) (Web Crawled Data)",
                            },
                            "e85d865943ee6d8369307569d2ad9de0": {
                                "count": 68,
                                "label": "Acuris Risk Intelligence Adverse Media Data",
                            },
                            "b6382672c6741fe1bca28d2668c1732b": {
                                "count": 216,
                                "label": "China LHNB MOFCOM Foreign Investment Directory",
                            },
                            "48c4f3b0f7fcc732c9d075893ad004d2": {
                                "count": 10,
                                "label": "Hong Kong Stock Exchange - Listed Company Directors",
                            },
                            "db3416894cd5f0c4d2d6ecc79bdaf366": {
                                "count": 1065,
                                "label": "EDI Publicly-Listed Global Security Issuers (3rd Party Data)",
                            },
                            "faa9caafcfabcee04ef2f0b21dd9197a": {
                                "count": 155,
                                "label": "South Korea Imports & Exports (2021 - Present)",
                            },
                            "b812677c0a32a1746b3ac741c7b97ae0": {
                                "count": 71620,
                                "label": "Legal Entity Identifier (LEI) Registry (3rd Party Data)",
                            },
                            "0ff02c63234d4447c803acd7748b9afc": {
                                "count": 28,
                                "label": "China Company Directory (Shuidi)(Web Crawled Data)",
                            },
                            "de2a65257042beae5373354223cbd55b": {
                                "count": 3,
                                "label": "Japan Houjin National Tax Agency Corporation Number Publication Site",
                            },
                            "2cd05c04774a433ce4d79e61ee105d08": {
                                "count": 9,
                                "label": "Hong Kong Judiciary Judgments",
                            },
                            "ce462e9deea545cce35df38c48512a0c": {
                                "count": 1636,
                                "label": "India Imports & Exports (January 2023 - Present)",
                            },
                            "8de630b7a702183da138321ae0f1c4b0": {
                                "count": 1,
                                "label": "China Imports & Exports (2022 - Present)",
                            },
                            "ecdfb3f2ecc8c3797e77d5795a8066ef": {
                                "count": 5,
                                "label": "UK Corporate Registry",
                            },
                            "eaa2c4801c4acde073decdfae533bd0e": {
                                "count": 4,
                                "label": "France BODACC",
                            },
                            "8f50655ba1d1552ab4b89d119bd9c318": {
                                "count": 3822,
                                "label": "China Trademarks & Intellectual Property System",
                            },
                            "441b8bd29977208d312f2192675b05d9": {
                                "count": 1,
                                "label": "China Company Directory (Shunqiwang) (Web Crawled Data)",
                            },
                            "0cf0044442daf17258e27a6ddd49770f": {
                                "count": 225,
                                "label": "China NECIPS",
                            },
                            "93f0d99bcf2cc0b9f09cacc92640e54c": {
                                "count": 1,
                                "label": "Zimbabwe Imports & Exports (January 2023 - June 2023)",
                            },
                            "ddbf93a5c5d568ccdb0c2455f7ecbfc8": {
                                "count": 9,
                                "label": "USA CorpWatch SEC 10-K Exhibit 21 Database (3rd Party Data)",
                            },
                            "148946e7e3e5b2ba031bfcefa28e4d83": {
                                "count": 33,
                                "label": "China Company Directory (Aiqicha)(Web Crawled Data)",
                            },
                            "e85f886caf11a16c40512647def393e0": {
                                "count": 20,
                                "label": "New Zealand Corporate Registry",
                            },
                            "16a4cc2d0f467fa993b28587d542a25d": {
                                "count": 11,
                                "label": "USA Imports (2021 - Present)",
                            },
                            "546a239179eb5ba24177e29e308005bf": {
                                "count": 2,
                                "label": "Pakistan Imports (2011-2021)",
                            },
                            "7e0b45866bdcca61b2cfd455e5403dc2": {
                                "count": 9,
                                "label": "China Central Government Procurement Center Enterprise Database",
                            },
                            "b67ab545f3ddc960d272d11ec5952665": {
                                "count": 6,
                                "label": "France Sirene/Infogreffe Commercial Registries",
                            },
                            "74cf0fc2a62f9c8f4e88f8a0b3ffcca4": {
                                "count": 2,
                                "label": "Hong Kong Companies Registry",
                            },
                            "4ea8bac1bed868e1510ffd21842e9551": {
                                "count": 39,
                                "label": "UK Persons with Significant Control",
                            },
                            "8f21460f1cc54773b9672cb5efdb01cf": {
                                "count": 7,
                                "label": "China Ministry of Finance Government Procurement Announcements",
                            },
                        },
                        "risk": {
                            "regulatory_action": {
                                "value": true,
                                "metadata": {},
                                "level": "high",
                            },
                            "eu_high_risk_third": {
                                "value": true,
                                "metadata": {"country": ["PHL", "AFG"]},
                                "level": "relevant",
                            },
                            "reputational_risk_financial_crime": {
                                "value": true,
                                "metadata": {},
                                "level": "elevated",
                            },
                            "owner_of_sanctioned_entity": {
                                "value": 3,
                                "metadata": {},
                                "level": "high",
                            },
                            "basel_aml": {
                                "value": 8.45,
                                "metadata": {"country": ["AFG"]},
                                "level": "relevant",
                            },
                            "state_owned": {
                                "value": true,
                                "metadata": {},
                                "level": "high",
                            },
                            "owner_of_export_controls_entity": {
                                "value": 3,
                                "metadata": {},
                                "level": "high",
                            },
                            "meu_list_contractors": {
                                "value": true,
                                "metadata": {
                                    "sources": [
                                        "China Central Government Procurement Center Enterprise Database",
                                        "China Ministry of Finance Government Procurement Announcements",
                                    ]
                                },
                                "level": "high",
                            },
                            "reputational_risk_other": {
                                "value": true,
                                "metadata": {},
                                "level": "elevated",
                            },
                            "owner_of_sheffield_hallam_university_reports_forced_labor_entity": {
                                "value": 3,
                                "metadata": {},
                                "level": "high",
                            },
                            "cpi_score": {
                                "value": 16,
                                "metadata": {"country": ["AFG"]},
                                "level": "relevant",
                            },
                            "pep_adjacent": {
                                "value": true,
                                "metadata": {},
                                "level": "elevated",
                            },
                            "owner_of_forced_labor_xinjiang_entity": {
                                "value": 1,
                                "metadata": {},
                                "level": "high",
                            },
                            "law_enforcement_action": {
                                "value": true,
                                "metadata": {},
                                "level": "elevated",
                            },
                        },
                        "user_attribute_counts": {},
                        "user_attribute_count": {},
                        "user_record_count": 0,
                        "user_related_entities_count": 0,
                        "user_relationship_count": {},
                        "related_entities_count": 22179,
                        "attribute_counts": {
                            "company_type": 20,
                            "name": 25,
                            "business_purpose": 41,
                            "identifier": 33,
                            "additional_information": 29,
                            "country": 35,
                            "contact": 49,
                            "shares": 11,
                            "status": 6,
                            "address": 91,
                            "financials": 7,
                        },
                        "attribute_count": {
                            "company_type": 20,
                            "name": 25,
                            "business_purpose": 41,
                            "identifier": 33,
                            "additional_information": 29,
                            "country": 35,
                            "contact": 49,
                            "shares": 11,
                            "status": 6,
                            "address": 91,
                            "financials": 7,
                        },
                        "reference_id": "bc5bfbbd56e094337aa743fa721a48cc/firm_fffe1851b4a2a8c09c822409757dd675.html/1553604848901:20b21074f749bb4124997622fe6f3808",
                    },
                    "type": "about",
                }
            ],
        ),
    )
    """

    references: RecordReferences

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import Coordinates, EntitySearchResponse, FilterList, RecordSearchResponse, SearchResults, SourceId

__all__ = ["Coordinates", "EntitySearchResponse", "FilterList", "RecordSearchResponse", "SearchResults", "SourceId"]



# This file was auto-generated by Fern from our API Definition.

from .entity_summary_response import EntitySummaryResponse
from .get_entity_response import GetEntityResponse

__all__ = ["EntitySummaryResponse", "GetEntityResponse"]



# This file was auto-generated by Fern from our API Definition.

from . import (
    attributes,
    auth,
    base_types,
    entity,
    generated_types,
    info,
    notifications,
    project,
    record,
    resolution,
    resource,
    search,
    shared_errors,
    shared_types,
    source,
    supply_chain,
    trade,
    traversal,
)
from .attributes import AddAttribute, AttributeProperties, AttributeResponse, AttributeResponseData, UpdateAttribute
from .auth import AuthResponse
from .base_types import CountQualifier, PaginatedResponse, QualifiedCount
from .entity import EntitySummaryResponse, GetEntityResponse
from .environment import SayariEnvironment
from .generated_types import (
    AdditionalInformationData,
    AdditionalInformationInfo,
    AdditionalInformationProperties,
    AddressData,
    AddressInfo,
    AddressProperties,
    AddressType,
    AttributeData,
    AttributeDetails,
    Attributes,
    BothIdentifierTypes,
    BusinessPurposeData,
    BusinessPurposeInfo,
    BusinessPurposeProperties,
    BusinessPurposeStandard,
    CompanyStatus,
    CompanyTypeData,
    CompanyTypeInfo,
    CompanyTypeProperties,
    ContactData,
    ContactInfo,
    ContactProperties,
    ContactType,
    Country,
    CountryContext,
    CountryData,
    CountryInfo,
    CountryProperties,
    Currency,
    DateOfBirthData,
    DateOfBirthInfo,
    DateOfBirthProperties,
    Entities,
    FinanceType,
    FinancesData,
    FinancesInfo,
    FinancesProperties,
    FinancialsData,
    FinancialsInfo,
    FinancialsProperties,
    Gender,
    GenderData,
    GenderInfo,
    GenderProperties,
    GenericData,
    GenericInfo,
    GenericProperties,
    IdentifierData,
    IdentifierInfo,
    IdentifierProperties,
    IdentifierType,
    Language,
    MeasurementData,
    MeasurementInfo,
    MeasurementProperties,
    MeasurementType,
    MonetaryValueContext,
    MonetaryValueData,
    MonetaryValueInfo,
    MonetaryValueProperties,
    NameContext,
    NameData,
    NameInfo,
    NameProperties,
    PersonStatus,
    PersonStatusData,
    PersonStatusInfo,
    PersonStatusProperties,
    PositionData,
    PositionInfo,
    PositionProperties,
    Relationships,
    Risk,
    RiskIntelligenceData,
    RiskIntelligenceInfo,
    RiskIntelligenceProperties,
    SharesData,
    SharesInfo,
    SharesProperties,
    StatusContext,
    StatusData,
    StatusInfo,
    StatusProperties,
    Tag,
    TranslatedNameData,
    TranslatedNameInfo,
    TranslatedNameProperties,
    TranslationContext,
    Unit,
    WeakIdentifierData,
    WeakIdentifierInfo,
    WeakIdentifierProperties,
    WeakIdentifierType,
)
from .info import EventInfo, HistoryInfo, HistoryResponse, UsageInfo, UsageResponse
from .notifications import (
    Notification,
    NotificationAdditionalInformation,
    NotificationType,
    ProjectNotificationData,
    ProjectNotificationsResponse,
    ResourceNotificationData,
    ResourceNotificationsResponse,
)
from .project import (
    BucketAgg,
    CreateProjectRequest,
    CreateProjectResponse,
    DeleteProjectResponse,
    DocCount,
    GetProjectEntitiesAcceptHeader,
    GetProjectEntitiesResponse,
    GetProjectsResponse,
    HsCodeAgg,
    HsCodeAggBucket,
    HsCodeAggTerms,
    IntKeyValue,
    Project,
    ProjectCounts,
    ProjectEntitiesAggs,
    ProjectEntitiesAggsDefinition,
    ProjectEntitiesFilter,
    ProjectEntity,
    ProjectEntityUpstream,
    ProjectShareOnCreate,
    ProjectWithMembers,
    PsaSummary,
    Role,
    RoleMember,
    RoleMemberType,
    SortField,
    TradeCount,
)
from .record import GetRecordResponse, RecordReferences
from .resolution import MatchExplanation, MatchStrength, ResolutionResponse, ResolutionResponseFields, ResolutionResult
from .resource import DeleteResourceResponse, EntityResponseData, ResourceType, SaveEntityRequest, SaveEntityResponse
from .search import Coordinates, EntitySearchResponse, FilterList, RecordSearchResponse, SearchResults, SourceId
from .shared_errors import (
    BadGateway,
    BadGatewayResponse,
    BadRequest,
    BadRequestResponse,
    ConnectionError,
    ConnectionErrorResponse,
    InternalServerError,
    InternalServerErrorResponse,
    MethodNotAllowed,
    MethodNotAllowedResponse,
    NotAcceptable,
    NotAcceptableResponse,
    NotFound,
    NotFoundResponse,
    RateLimitExceeded,
    RateLimitResponse,
    Unauthorized,
    UnauthorizedResponse,
)
from .shared_types import (
    ClientName,
    CompanyType,
    Coordinate,
    CoreEntity,
    EmbeddedEntity,
    EntityDetails,
    EntityHsCode,
    EntityMatches,
    EntityRegistrationDate,
    EntityRelationships,
    EntityRisk,
    EntitySummary,
    EntityTranslatedLabel,
    Identifier,
    PossiblySameAs,
    PossiblySameAsData,
    PossiblySameAsMatch,
    Psa,
    PsaEntity,
    PsaMatchKeys,
    RecordDetails,
    ReferencedBy,
    ReferencedByData,
    ReferencedByDataType,
    RelationshipCount,
    RelationshipData,
    RelationshipInfo,
    RiskData,
    RiskLevel,
    RiskValue,
    SearchField,
    ShipmentArrival,
    ShipmentDeparture,
    SourceCountInfo,
    Status,
)
from .source import GetSourceResponse, ListSourcesResponse, Source
from .supply_chain import (
    EntityId,
    TradeTraversalEntity,
    TradeTraversalPath,
    TradeTraversalPathOrSegment,
    TradeTraversalPathSegment,
    TradeTraversalPathSegments,
    TradeTraversalProduct,
    UpstreamTradeTraversalResponse,
)
from .trade import (
    BusinessPurpose,
    BuyerSearchResponse,
    DataSource,
    HsCode,
    HsCodeInfo,
    MonetaryValue,
    Shipment,
    ShipmentAddress,
    ShipmentCountry,
    ShipmentIdentifier,
    ShipmentMetadata,
    ShipmentSearchResponse,
    SourceOrDestinationEntity,
    SupplierMetadata,
    SupplierOrBuyer,
    SupplierSearchResponse,
    TradeFilterList,
    Weight,
)
from .traversal import (
    ShortestPathData,
    ShortestPathResponse,
    TraversalData,
    TraversalPath,
    TraversalRelationshipData,
    TraversalResponse,
)
from .version import __version__

__all__ = [
    "AddAttribute",
    "AdditionalInformationData",
    "AdditionalInformationInfo",
    "AdditionalInformationProperties",
    "AddressData",
    "AddressInfo",
    "AddressProperties",
    "AddressType",
    "AttributeData",
    "AttributeDetails",
    "AttributeProperties",
    "AttributeResponse",
    "AttributeResponseData",
    "Attributes",
    "AuthResponse",
    "BadGateway",
    "BadGatewayResponse",
    "BadRequest",
    "BadRequestResponse",
    "BothIdentifierTypes",
    "BucketAgg",
    "BusinessPurpose",
    "BusinessPurposeData",
    "BusinessPurposeInfo",
    "BusinessPurposeProperties",
    "BusinessPurposeStandard",
    "BuyerSearchResponse",
    "ClientName",
    "CompanyStatus",
    "CompanyType",
    "CompanyTypeData",
    "CompanyTypeInfo",
    "CompanyTypeProperties",
    "ConnectionError",
    "ConnectionErrorResponse",
    "ContactData",
    "ContactInfo",
    "ContactProperties",
    "ContactType",
    "Coordinate",
    "Coordinates",
    "CoreEntity",
    "CountQualifier",
    "Country",
    "CountryContext",
    "CountryData",
    "CountryInfo",
    "CountryProperties",
    "CreateProjectRequest",
    "CreateProjectResponse",
    "Currency",
    "DataSource",
    "DateOfBirthData",
    "DateOfBirthInfo",
    "DateOfBirthProperties",
    "DeleteProjectResponse",
    "DeleteResourceResponse",
    "DocCount",
    "EmbeddedEntity",
    "Entities",
    "EntityDetails",
    "EntityHsCode",
    "EntityId",
    "EntityMatches",
    "EntityRegistrationDate",
    "EntityRelationships",
    "EntityResponseData",
    "EntityRisk",
    "EntitySearchResponse",
    "EntitySummary",
    "EntitySummaryResponse",
    "EntityTranslatedLabel",
    "EventInfo",
    "FilterList",
    "FinanceType",
    "FinancesData",
    "FinancesInfo",
    "FinancesProperties",
    "FinancialsData",
    "FinancialsInfo",
    "FinancialsProperties",
    "Gender",
    "GenderData",
    "GenderInfo",
    "GenderProperties",
    "GenericData",
    "GenericInfo",
    "GenericProperties",
    "GetEntityResponse",
    "GetProjectEntitiesAcceptHeader",
    "GetProjectEntitiesResponse",
    "GetProjectsResponse",
    "GetRecordResponse",
    "GetSourceResponse",
    "HistoryInfo",
    "HistoryResponse",
    "HsCode",
    "HsCodeAgg",
    "HsCodeAggBucket",
    "HsCodeAggTerms",
    "HsCodeInfo",
    "Identifier",
    "IdentifierData",
    "IdentifierInfo",
    "IdentifierProperties",
    "IdentifierType",
    "IntKeyValue",
    "InternalServerError",
    "InternalServerErrorResponse",
    "Language",
    "ListSourcesResponse",
    "MatchExplanation",
    "MatchStrength",
    "MeasurementData",
    "MeasurementInfo",
    "MeasurementProperties",
    "MeasurementType",
    "MethodNotAllowed",
    "MethodNotAllowedResponse",
    "MonetaryValue",
    "MonetaryValueContext",
    "MonetaryValueData",
    "MonetaryValueInfo",
    "MonetaryValueProperties",
    "NameContext",
    "NameData",
    "NameInfo",
    "NameProperties",
    "NotAcceptable",
    "NotAcceptableResponse",
    "NotFound",
    "NotFoundResponse",
    "Notification",
    "NotificationAdditionalInformation",
    "NotificationType",
    "PaginatedResponse",
    "PersonStatus",
    "PersonStatusData",
    "PersonStatusInfo",
    "PersonStatusProperties",
    "PositionData",
    "PositionInfo",
    "PositionProperties",
    "PossiblySameAs",
    "PossiblySameAsData",
    "PossiblySameAsMatch",
    "Project",
    "ProjectCounts",
    "ProjectEntitiesAggs",
    "ProjectEntitiesAggsDefinition",
    "ProjectEntitiesFilter",
    "ProjectEntity",
    "ProjectEntityUpstream",
    "ProjectNotificationData",
    "ProjectNotificationsResponse",
    "ProjectShareOnCreate",
    "ProjectWithMembers",
    "Psa",
    "PsaEntity",
    "PsaMatchKeys",
    "PsaSummary",
    "QualifiedCount",
    "RateLimitExceeded",
    "RateLimitResponse",
    "RecordDetails",
    "RecordReferences",
    "RecordSearchResponse",
    "ReferencedBy",
    "ReferencedByData",
    "ReferencedByDataType",
    "RelationshipCount",
    "RelationshipData",
    "RelationshipInfo",
    "Relationships",
    "ResolutionResponse",
    "ResolutionResponseFields",
    "ResolutionResult",
    "ResourceNotificationData",
    "ResourceNotificationsResponse",
    "ResourceType",
    "Risk",
    "RiskData",
    "RiskIntelligenceData",
    "RiskIntelligenceInfo",
    "RiskIntelligenceProperties",
    "RiskLevel",
    "RiskValue",
    "Role",
    "RoleMember",
    "RoleMemberType",
    "SaveEntityRequest",
    "SaveEntityResponse",
    "SayariEnvironment",
    "SearchField",
    "SearchResults",
    "SharesData",
    "SharesInfo",
    "SharesProperties",
    "Shipment",
    "ShipmentAddress",
    "ShipmentArrival",
    "ShipmentCountry",
    "ShipmentDeparture",
    "ShipmentIdentifier",
    "ShipmentMetadata",
    "ShipmentSearchResponse",
    "ShortestPathData",
    "ShortestPathResponse",
    "SortField",
    "Source",
    "SourceCountInfo",
    "SourceId",
    "SourceOrDestinationEntity",
    "Status",
    "StatusContext",
    "StatusData",
    "StatusInfo",
    "StatusProperties",
    "SupplierMetadata",
    "SupplierOrBuyer",
    "SupplierSearchResponse",
    "Tag",
    "TradeCount",
    "TradeFilterList",
    "TradeTraversalEntity",
    "TradeTraversalPath",
    "TradeTraversalPathOrSegment",
    "TradeTraversalPathSegment",
    "TradeTraversalPathSegments",
    "TradeTraversalProduct",
    "TranslatedNameData",
    "TranslatedNameInfo",
    "TranslatedNameProperties",
    "TranslationContext",
    "TraversalData",
    "TraversalPath",
    "TraversalRelationshipData",
    "TraversalResponse",
    "Unauthorized",
    "UnauthorizedResponse",
    "Unit",
    "UpdateAttribute",
    "UpstreamTradeTraversalResponse",
    "UsageInfo",
    "UsageResponse",
    "WeakIdentifierData",
    "WeakIdentifierInfo",
    "WeakIdentifierProperties",
    "WeakIdentifierType",
    "Weight",
    "__version__",
    "attributes",
    "auth",
    "base_types",
    "entity",
    "generated_types",
    "info",
    "notifications",
    "project",
    "record",
    "resolution",
    "resource",
    "search",
    "shared_errors",
    "shared_types",
    "source",
    "supply_chain",
    "trade",
    "traversal",
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.query_encoder import encode_query
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.auth_response import AuthResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthResponse:
        """
        Hit the auth endpoint to get a bearer token

        Parameters
        ----------
        client_id : str

        client_secret : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthResponse
            OK

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.auth.get_token(
            client_id="your client_id here",
            client_secret="your client_secret here",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "oauth/token"),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            json=jsonable_encoder(
                {
                    "client_id": client_id,
                    "client_secret": client_secret,
                    "audience": "sayari.com",
                    "grant_type": "client_credentials",
                }
            )
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(
                    {
                        "client_id": client_id,
                        "client_secret": client_secret,
                        "audience": "sayari.com",
                        "grant_type": "client_credentials",
                    }
                ),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AuthResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthResponse:
        """
        Hit the auth endpoint to get a bearer token

        Parameters
        ----------
        client_id : str

        client_secret : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthResponse
            OK

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.auth.get_token(
            client_id="your client_id here",
            client_secret="your client_secret here",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "oauth/token"),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            json=jsonable_encoder(
                {
                    "client_id": client_id,
                    "client_secret": client_secret,
                    "audience": "sayari.com",
                    "grant_type": "client_credentials",
                }
            )
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(
                    {
                        "client_id": client_id,
                        "client_secret": client_secret,
                        "audience": "sayari.com",
                        "grant_type": "client_credentials",
                    }
                ),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AuthResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.query_encoder import encode_query
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_gateway import BadGateway
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.connection_error import ConnectionError
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_gateway_response import BadGatewayResponse
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.connection_error_response import ConnectionErrorResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.project_notifications_response import ProjectNotificationsResponse
from .types.resource_notifications_response import ResourceNotificationsResponse


class NotificationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def project_notifications(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectNotificationsResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> The Project Notifications endpoint returns a list of notifications on all entities saved to a project.

        Parameters
        ----------
        id : str
            Unique identifier of the project

        limit : typing.Optional[int]
            Limit total notifications in the response. Defaults to 100.

        offset : typing.Optional[int]
            Offset which notifications are returned. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectNotificationsResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.notifications.project_notifications(
            id="0oZnoG",
            limit=20,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/projects/{jsonable_encoder(id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectNotificationsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def resource_notifications(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResourceNotificationsResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> The Resource Notifications endpoint returns a list of notifications for a saved entity.

        Parameters
        ----------
        id : str
            Unique identifier of the resource

        limit : typing.Optional[int]
            Limit total notifications in the response. Defaults to 100.

        offset : typing.Optional[int]
            Offset which notifications are returned. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResourceNotificationsResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.notifications.resource_notifications(
            id="03ePyj",
            limit=20,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/resources/{jsonable_encoder(id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ResourceNotificationsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_project_notifications(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes all notifications from a project.

        Parameters
        ----------
        project_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.notifications.delete_project_notifications(
            project_id="YWmNKV",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/projects/{jsonable_encoder(project_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_entity_notifications(
        self, entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes notifications for saved resources of an entity.

        Parameters
        ----------
        entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.notifications.delete_entity_notifications(
            entity_id="N0xLDy4wcud-M1ZtwdsvRA",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/entity/{jsonable_encoder(entity_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_resource_notifications(
        self, resource_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes notifications for a saved resource.

        Parameters
        ----------
        resource_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.notifications.delete_resource_notifications(
            resource_id="oGxxqG",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/resources/{jsonable_encoder(resource_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncNotificationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def project_notifications(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectNotificationsResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> The Project Notifications endpoint returns a list of notifications on all entities saved to a project.

        Parameters
        ----------
        id : str
            Unique identifier of the project

        limit : typing.Optional[int]
            Limit total notifications in the response. Defaults to 100.

        offset : typing.Optional[int]
            Offset which notifications are returned. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectNotificationsResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.notifications.project_notifications(
            id="0oZnoG",
            limit=20,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/projects/{jsonable_encoder(id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ProjectNotificationsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def resource_notifications(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResourceNotificationsResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> The Resource Notifications endpoint returns a list of notifications for a saved entity.

        Parameters
        ----------
        id : str
            Unique identifier of the resource

        limit : typing.Optional[int]
            Limit total notifications in the response. Defaults to 100.

        offset : typing.Optional[int]
            Offset which notifications are returned. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResourceNotificationsResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.notifications.resource_notifications(
            id="03ePyj",
            limit=20,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/resources/{jsonable_encoder(id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "limit": limit,
                            "offset": offset,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ResourceNotificationsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_project_notifications(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes all notifications from a project.

        Parameters
        ----------
        project_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.notifications.delete_project_notifications(
            project_id="YWmNKV",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/projects/{jsonable_encoder(project_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_entity_notifications(
        self, entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes notifications for saved resources of an entity.

        Parameters
        ----------
        entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.notifications.delete_entity_notifications(
            entity_id="N0xLDy4wcud-M1ZtwdsvRA",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/entity/{jsonable_encoder(entity_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_resource_notifications(
        self, resource_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes notifications for a saved resource.

        Parameters
        ----------
        resource_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.notifications.delete_resource_notifications(
            resource_id="oGxxqG",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/notifications/resources/{jsonable_encoder(resource_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from .trade_traversal_path import TradeTraversalPath
from .trade_traversal_path_segment import TradeTraversalPathSegment

TradeTraversalPathOrSegment = typing.Union[
    typing.List[TradeTraversalPath], typing.List[typing.List[TradeTraversalPathSegment]]
]



# This file was auto-generated by Fern from our API Definition.

import typing

Currency = typing.Union[
    typing.Literal[
        "AFN",
        "EUR",
        "ALL",
        "DZD",
        "USD",
        "AOA",
        "XCD",
        "ARS",
        "AMD",
        "AWG",
        "AUD",
        "AZN",
        "BSD",
        "BHD",
        "BDT",
        "BBD",
        "BYN",
        "BZD",
        "XOF",
        "BMD",
        "INR",
        "BTN",
        "BOB",
        "BOV",
        "BAM",
        "BWP",
        "NOK",
        "BRL",
        "BND",
        "BGN",
        "BIF",
        "CVE",
        "KHR",
        "XAF",
        "CAD",
        "KYD",
        "CLP",
        "CLF",
        "CNY",
        "COP",
        "COU",
        "KMF",
        "CDF",
        "NZD",
        "CRC",
        "HRK",
        "CUP",
        "CUC",
        "ANG",
        "CZK",
        "DKK",
        "DJF",
        "DOP",
        "EGP",
        "SVC",
        "ERN",
        "ETB",
        "FKP",
        "FJD",
        "XPF",
        "GMD",
        "GEL",
        "GHS",
        "GIP",
        "GTQ",
        "GBP",
        "GNF",
        "GYD",
        "HTG",
        "HNL",
        "HKD",
        "HUF",
        "ISK",
        "IDR",
        "XDR",
        "IRR",
        "IQD",
        "ILS",
        "JMD",
        "JPY",
        "JOD",
        "KZT",
        "KES",
        "KPW",
        "KRW",
        "KWD",
        "KGS",
        "LAK",
        "LBP",
        "LSL",
        "ZAR",
        "LRD",
        "LYD",
        "CHF",
        "MOP",
        "MKD",
        "MGA",
        "MWK",
        "MYR",
        "MVR",
        "MRU",
        "MUR",
        "XUA",
        "MXN",
        "MXV",
        "MDL",
        "MNT",
        "MAD",
        "MZN",
        "MMK",
        "NAD",
        "NPR",
        "NIO",
        "NGN",
        "OMR",
        "PKR",
        "PAB",
        "PGK",
        "PYG",
        "PEN",
        "PHP",
        "PLN",
        "QAR",
        "RON",
        "RUB",
        "RWF",
        "SHP",
        "WST",
        "STN",
        "SAR",
        "RSD",
        "SCR",
        "SLL",
        "SGD",
        "XSU",
        "SBD",
        "SOS",
        "SSP",
        "LKR",
        "SDG",
        "SRD",
        "SZL",
        "SEK",
        "CHE",
        "CHW",
        "SYP",
        "TWD",
        "TJS",
        "TZS",
        "THB",
        "TOP",
        "TTD",
        "TND",
        "TRY",
        "TMT",
        "UGX",
        "UAH",
        "AED",
        "USN",
        "UYU",
        "UYI",
        "UYW",
        "UZS",
        "VUV",
        "VES",
        "VND",
        "YER",
        "ZMW",
        "ZWL",
        "XBA",
        "XBB",
        "XBC",
        "XBD",
        "XTS",
        "XXX",
        "XAU",
        "XPD",
        "XPT",
        "XAG",
        "LVL",
        "SKK",
        "ITL",
        "NLG",
        "BEF",
        "MRO",
        "UKH",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...base_types.types.paginated_response import PaginatedResponse
from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .project_entities_aggs import ProjectEntitiesAggs
from .project_entity import ProjectEntity


class GetProjectEntitiesResponse(PaginatedResponse):
    """
    Examples
    --------
    from sayari import (
        BucketAgg,
        CoreEntity,
        GetProjectEntitiesResponse,
        HsCodeAgg,
        HsCodeAggBucket,
        HsCodeAggTerms,
        IntKeyValue,
        ProjectEntitiesAggs,
        ProjectEntity,
        ProjectEntityUpstream,
        PsaSummary,
        QualifiedCount,
        RiskData,
        SourceCountInfo,
        TradeCount,
    )

    GetProjectEntitiesResponse(
        limit=10000,
        size=QualifiedCount(
            count=1,
            qualifier="eq",
        ),
        data=[
            ProjectEntity(
                type="entity",
                id="GOeOE8",
                project="gPq6EY",
                label="RAMESH FLOWERS PRIVATE LIMITED",
                created="2024-03-08 18:52:59.41176+00",
                updated="2024-03-08 18:52:59.41176+00",
                updated_by="auth0|5e45bd8caccd890e68147513",
                version=0,
                entity_id="FyI9lrlmA-7z_3enlTOhlw",
                tag_ids=[],
                case_status="not_assigned",
                received_hs_codes=["271290", "271220"],
                shipped_hs_codes=[],
                trade_count_incl_mg=TradeCount(
                    receiver_of=5,
                    shipper_of=0,
                ),
                match_strength={
                    "rules": {
                        "is_strong_v1": false,
                        "any_v1": "",
                        "is_any_v1": false,
                        "strong_v1": "",
                        "is_weak_v1": false,
                        "weak_v1": "",
                        "strength_v1": "none",
                    }
                },
                upstream=ProjectEntityUpstream(
                    risk=[],
                    countries=[],
                ),
                psa=PsaSummary(
                    risk=[
                        "eu_high_risk_third",
                        "basel_aml",
                        "cpi_score",
                        "owner_of_regulatory_action_entity",
                    ],
                    countries=["USA", "CAN", "MAR", "BEL", "GEO"],
                    count=2,
                ),
                summary=CoreEntity(
                    id="FyI9lrlmA-7z_3enlTOhlw",
                    type="company",
                    label="RAMESH FLOWERS PRIVATE LIMITED",
                    identifiers=[],
                    addresses=[
                        "A62(A) SIPCOT INDUSTRIAL COMPLEX THERKU VEERAPANDIAPURAM P.O. TUTICORIN628002 TAMIL NADU INDIA GSTIN 33AAACR9251P1ZY IEC 3595005775 PANNO AAACR92",
                        "A62(A) SIPCOT INDUSTRIAL COMPLEX TUTICORIN628002 TAMIL NADU INDIA GSTIN 33AAACR9251P1ZY IEC 3595005775 PH914612340505 TO 2340509",
                    ],
                    countries=["IND"],
                    names=["RAMESH FLOWERS PRIVATE LIMITED"],
                    related_entities_count=8,
                    user_related_entities_count=0,
                    relationship_counts={
                        "notify_party_of": 5,
                        "receives_from": 3,
                        "receiver_of": 5,
                    },
                    user_relationship_counts={},
                    attribute_counts={"name": 1, "country": 1, "address": 2},
                    trade_count={"sent": 0, "received": 5},
                    user_attribute_counts={},
                    record_count=5,
                    user_record_count=0,
                    source_counts={
                        "faa9caafcfabcee04ef2f0b21dd9197a": SourceCountInfo(
                            count=10,
                            label="South Korea Imports & Exports (2021 - Present)",
                        )
                    },
                    risk={
                        "forced_labor_xinjiang_origin_subtier": RiskData(
                            value=3.0,
                            metadata={},
                            level="high",
                        ),
                        "basel_aml": RiskData(
                            value=4.95,
                            metadata={"country": ["IND"]},
                            level="relevant",
                        ),
                        "forced_labor_sheffield_hallam_university_reports_origin_subtier": RiskData(
                            value=3.0,
                            metadata={},
                            level="high",
                        ),
                        "cpi_score": RiskData(
                            value=40.0,
                            metadata={"country": ["IND"]},
                            level="relevant",
                        ),
                        "forced_labor_uflpa_origin_subtier": RiskData(
                            value=3.0,
                            metadata={},
                            level="high",
                        ),
                    },
                    attributes={
                        "address": [
                            {
                                "value": {
                                    "value": "A62(A) SIPCOT INDUSTRIAL COMPLEX THERKU VEERAPANDIAPURAM P.O. TUTICORIN628002 TAMIL NADU INDIA GSTIN 33AAACR9251P1ZY IEC 3595005775 PANNO AAACR92",
                                    "house": "Gstin 33aaacr9251p1zy",
                                    "house_number": "3595005775",
                                    "road": "Panno",
                                    "city": "NADU",
                                    "state": "Tamil Nadu",
                                    "postcode": "Aaacr92",
                                    "country": "India",
                                    "x": 74.99698,
                                    "y": 12.85316,
                                    "precision_code": "G3",
                                    "normalized": "33AAACR9251P1ZY 3595005775 A62A AAACR92 COMPLEX GSTIN IEC INDIA INDUSTRIAL NADU O P PANNO SIPCOT TAMIL THERKU TUTICORIN628002 VEERAPANDIAPURAM",
                                },
                                "properties": [
                                    {
                                        "record": "faa9caafcfabcee04ef2f0b21dd9197a/MAEU226738031/1693109172077",
                                        "record_count": 4,
                                        "editable": false,
                                    }
                                ],
                            },
                            {
                                "value": {
                                    "value": "A62(A) SIPCOT INDUSTRIAL COMPLEX TUTICORIN628002 TAMIL NADU INDIA GSTIN 33AAACR9251P1ZY IEC 3595005775 PH914612340505 TO 2340509",
                                    "house": "Gstin 33aaacr9251p1zy",
                                    "house_number": "3595005775",
                                    "road": "Ph914612340505 To 2340509",
                                    "city": "TO",
                                    "state": "Tamil Nadu",
                                    "postcode": "Tuticorin628002",
                                    "country": "India",
                                    "x": 77.06,
                                    "y": 20.00896,
                                    "precision_code": "G3",
                                    "normalized": "2340509 33AAACR9251P1ZY 3595005775 A62A COMPLEX GSTIN IEC INDIA INDUSTRIAL NADU PH914612340505 SIPCOT TAMIL TO TUTICORIN628002",
                                },
                                "properties": [
                                    {
                                        "record": "faa9caafcfabcee04ef2f0b21dd9197a/MAEU1KT860018/1693109343516",
                                        "record_count": 2,
                                        "editable": false,
                                    }
                                ],
                            },
                        ],
                        "country": [
                            {
                                "value": {
                                    "value": "IND",
                                    "context": "address",
                                    "Original Text": "India",
                                },
                                "properties": [
                                    {
                                        "record": "faa9caafcfabcee04ef2f0b21dd9197a/MAEU226738031/1693109172077",
                                        "record_count": 6,
                                        "editable": false,
                                    }
                                ],
                            }
                        ],
                        "name": [
                            {
                                "value": {
                                    "value": "RAMESH FLOWERS PRIVATE LIMITED",
                                    "context": "primary",
                                },
                                "properties": [
                                    {
                                        "record": "faa9caafcfabcee04ef2f0b21dd9197a/MAEU226738031/1693109172077",
                                        "record_count": 10,
                                        "editable": false,
                                    }
                                ],
                            }
                        ],
                    },
                ),
            )
        ],
        aggregations=ProjectEntitiesAggs(
            risk=[
                BucketAgg(
                    key="basel_aml",
                    doc_count=1,
                ),
                BucketAgg(
                    key="cpi_score",
                    doc_count=1,
                ),
                BucketAgg(
                    key="forced_labor_xinjiang_origin_subtier",
                    doc_count=1,
                ),
                BucketAgg(
                    key="forced_labor_uflpa_origin_subtier",
                    doc_count=1,
                ),
                BucketAgg(
                    key="forced_labor_sheffield_hallam_university_reports_origin_subtier",
                    doc_count=1,
                ),
            ],
            tag_ids=[],
            upstream_risk=[],
            hit_count=[
                BucketAgg(
                    key="hit_count",
                    doc_count=1,
                )
            ],
            country=[
                BucketAgg(
                    key="IND",
                    doc_count=1,
                    label="India",
                )
            ],
            shipment_counts=[
                BucketAgg(
                    key="receiver_of",
                    doc_count=5,
                    comment="Sum of all shipments of this type across matching documents",
                ),
                BucketAgg(
                    key="shipper_of",
                    doc_count=0,
                    comment="Sum of all shipments of this type across matching documents",
                ),
            ],
            source=[
                BucketAgg(
                    key="faa9caafcfabcee04ef2f0b21dd9197a",
                    label="South Korea Imports & Exports (2021 - Present)",
                    doc_count=1,
                )
            ],
            source_type=[
                BucketAgg(
                    key="trade_data",
                    doc_count=1,
                )
            ],
            region=[
                BucketAgg(
                    key="asia_pacific",
                    doc_count=1,
                )
            ],
            match_results=[
                BucketAgg(
                    key="sayari",
                    doc_count=1,
                )
            ],
            business_purpose=[],
            upstream_country=[],
            received_hs_codes=HsCodeAgg(
                doc_count=2,
                hs_code_terms=HsCodeAggTerms(
                    doc_count_error_upper_bound=0,
                    sum_other_doc_count=0,
                    buckets=[
                        HsCodeAggBucket(
                            key="271290",
                            doc_count=1,
                            hs_code_sums=IntKeyValue(
                                value=4,
                            ),
                        ),
                        HsCodeAggBucket(
                            key="271220",
                            doc_count=1,
                            hs_code_sums=IntKeyValue(
                                value=1,
                            ),
                        ),
                    ],
                ),
            ),
            case_statuses=[
                BucketAgg(
                    key="not_assigned",
                    doc_count=1,
                )
            ],
            shipped_hs_codes=HsCodeAgg(
                doc_count=0,
                hs_code_terms=HsCodeAggTerms(
                    doc_count_error_upper_bound=0,
                    sum_other_doc_count=0,
                    buckets=[],
                ),
            ),
        ),
    )
    """

    next: typing.Optional[str] = None
    prev: typing.Optional[str] = None
    data: typing.List[ProjectEntity]
    aggregations: ProjectEntitiesAggs

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.query_encoder import encode_query
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_gateway import BadGateway
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.connection_error import ConnectionError
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_gateway_response import BadGatewayResponse
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.connection_error_response import ConnectionErrorResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.add_attribute import AddAttribute
from .types.attribute_response import AttributeResponse
from .types.update_attribute import UpdateAttribute

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def post_attribute(
        self, *, request: AddAttribute, request_options: typing.Optional[RequestOptions] = None
    ) -> AttributeResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Adds a new Attribute

        Parameters
        ----------
        request : AddAttribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeResponse

        Examples
        --------
        from sayari import AddAttribute
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.attributes.post_attribute(
            request=AddAttribute(
                entity="zq04axX2dLn9tE6W6Q8Qhg",
                type="address",
                value={
                    "street1": "1600 Pennsylvania Avenue NW",
                    "city": "Washington DC",
                    "state": "Washington DC",
                    "postalCode": "20500",
                    "country": "US",
                },
                to_date="2024-04-30",
                from_date="2024-01-01",
                date="2024-02-15",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/attribute"),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AttributeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch_attribute(
        self, attribute_id: str, *, request: UpdateAttribute, request_options: typing.Optional[RequestOptions] = None
    ) -> AttributeResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Updates an existing Attribute

        Parameters
        ----------
        attribute_id : str

        request : UpdateAttribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeResponse

        Examples
        --------
        from sayari import UpdateAttribute
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.attributes.patch_attribute(
            attribute_id="enEwNGF4WDJkTG45dEU2VzZROFFoZ3xhZGRyZXNzfDBwbEVCMHxVNzhzN21yOUVFTThIZ3pwREM3UDFB",
            request=UpdateAttribute(
                value={
                    "street1": "1600 Pennsylvania Avenue NW",
                    "city": "Washington DC",
                    "state": "Washington DC",
                    "postalCode": "20500",
                    "country": "US",
                },
                to_date="2024-04-30",
                from_date="2024-01-01",
                date="2024-02-15",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="PATCH",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/attribute/{jsonable_encoder(attribute_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AttributeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_attribute(
        self, attribute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AttributeResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Delete an existing Attribute

        Parameters
        ----------
        attribute_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeResponse

        Examples
        --------
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.attributes.delete_attribute(
            attribute_id="enEwNGF4WDJkTG45dEU2VzZROFFoZ3xhZGRyZXNzfDBwbEVCMHxVNzhzN21yOUVFTThIZ3pwREM3UDFB",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/attribute/{jsonable_encoder(attribute_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AttributeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def post_attribute(
        self, *, request: AddAttribute, request_options: typing.Optional[RequestOptions] = None
    ) -> AttributeResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Adds a new Attribute

        Parameters
        ----------
        request : AddAttribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeResponse

        Examples
        --------
        from sayari import AddAttribute
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.attributes.post_attribute(
            request=AddAttribute(
                entity="zq04axX2dLn9tE6W6Q8Qhg",
                type="address",
                value={
                    "street1": "1600 Pennsylvania Avenue NW",
                    "city": "Washington DC",
                    "state": "Washington DC",
                    "postalCode": "20500",
                    "country": "US",
                },
                to_date="2024-04-30",
                from_date="2024-01-01",
                date="2024-02-15",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/attribute"),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AttributeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch_attribute(
        self, attribute_id: str, *, request: UpdateAttribute, request_options: typing.Optional[RequestOptions] = None
    ) -> AttributeResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Updates an existing Attribute

        Parameters
        ----------
        attribute_id : str

        request : UpdateAttribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeResponse

        Examples
        --------
        from sayari import UpdateAttribute
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.attributes.patch_attribute(
            attribute_id="enEwNGF4WDJkTG45dEU2VzZROFFoZ3xhZGRyZXNzfDBwbEVCMHxVNzhzN21yOUVFTThIZ3pwREM3UDFB",
            request=UpdateAttribute(
                value={
                    "street1": "1600 Pennsylvania Avenue NW",
                    "city": "Washington DC",
                    "state": "Washington DC",
                    "postalCode": "20500",
                    "country": "US",
                },
                to_date="2024-04-30",
                from_date="2024-01-01",
                date="2024-02-15",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="PATCH",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/attribute/{jsonable_encoder(attribute_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AttributeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_attribute(
        self, attribute_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AttributeResponse:
        """
        <Warning>This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Warning> Delete an existing Attribute

        Parameters
        ----------
        attribute_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeResponse

        Examples
        --------
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.attributes.delete_attribute(
            attribute_id="enEwNGF4WDJkTG45dEU2VzZROFFoZ3xhZGRyZXNzfDBwbEVCMHxVNzhzN21yOUVFTThIZ3pwREM3UDFB",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="DELETE",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/attribute/{jsonable_encoder(attribute_id)}"
            ),
            params=encode_query(
                jsonable_encoder(
                    request_options.get("additional_query_parameters") if request_options is not None else None
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AttributeResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        if _response.status_code == 502:
            raise BadGateway(pydantic_v1.parse_obj_as(BadGatewayResponse, _response.json()))  # type: ignore
        if _response.status_code == 520:
            raise ConnectionError(pydantic_v1.parse_obj_as(ConnectionErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .entity_id import EntityId
from .trade_traversal_entity import TradeTraversalEntity
from .trade_traversal_path_or_segment import TradeTraversalPathOrSegment


class UpstreamTradeTraversalResponse(pydantic_v1.BaseModel):
    """
    Examples
    --------
    from sayari import (
        TradeTraversalEntity,
        TradeTraversalPath,
        TradeTraversalPathSegment,
        TradeTraversalPathSegments,
        TradeTraversalProduct,
        UpstreamTradeTraversalResponse,
    )

    UpstreamTradeTraversalResponse(
        paths=[
            TradeTraversalPath(
                start="ESkH7J-UCRfY5t0_JXIH3w",
                end="A84F35yJ_3FT_hVzD9EzAw",
                paths=[
                    TradeTraversalPathSegments(
                        segments=[
                            TradeTraversalPathSegment(
                                src="A84F35yJ_3FT_hVzD9EzAw",
                                dst="ESkH7J-UCRfY5t0_JXIH3w",
                                products=[
                                    TradeTraversalProduct(
                                        hs_code="3206",
                                        min_date="2023-01-01",
                                        max_date="2023-11-25",
                                        arrival_countries=["VNM"],
                                        departure_countries=["IND"],
                                    ),
                                    TradeTraversalProduct(
                                        hs_code="3204",
                                        min_date="2023-03-01",
                                        max_date="2023-11-25",
                                        arrival_countries=["VNM"],
                                        departure_countries=["IND"],
                                    ),
                                ],
                            )
                        ],
                    )
                ],
            )
        ],
        entities={
            "ESkH7J-UCRfY5t0_JXIH3w": TradeTraversalEntity(
                type="company",
                label="ACUMEN HOUSEWARE INDUSTRY VIETNAM CO LTD",
                countries=["VNM"],
                risks=[],
            ),
            "A84F35yJ_3FT_hVzD9EzAw": TradeTraversalEntity(
                type="company",
                label="TOYO INK INDIA PRIVATE LIMITED",
                countries=["IND"],
                risks=[
                    "forced_labor_sheffield_hallam_university_reports_origin_subtier",
                    "forced_labor_uflpa_origin_subtier",
                    "forced_labor_xinjiang_origin_subtier",
                ],
            ),
        },
    )
    """

    status: typing.Optional[int] = None
    success: typing.Optional[bool] = None
    message: typing.Optional[str] = None
    entities: typing.Dict[EntityId, TradeTraversalEntity]
    paths: TradeTraversalPathOrSegment

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .entity_id import EntityId
from .trade_traversal_entity import TradeTraversalEntity
from .trade_traversal_path import TradeTraversalPath
from .trade_traversal_path_or_segment import TradeTraversalPathOrSegment
from .trade_traversal_path_segment import TradeTraversalPathSegment
from .trade_traversal_path_segments import TradeTraversalPathSegments
from .trade_traversal_product import TradeTraversalProduct
from .upstream_trade_traversal_response import UpstreamTradeTraversalResponse

__all__ = [
    "EntityId",
    "TradeTraversalEntity",
    "TradeTraversalPath",
    "TradeTraversalPathOrSegment",
    "TradeTraversalPathSegment",
    "TradeTraversalPathSegments",
    "TradeTraversalProduct",
    "UpstreamTradeTraversalResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from ...generated_types.types.country import Country
from ...generated_types.types.entities import Entities
from .company_type import CompanyType
from .entity_hs_code import EntityHsCode
from .entity_registration_date import EntityRegistrationDate
from .entity_risk import EntityRisk
from .entity_summary import EntitySummary
from .entity_translated_label import EntityTranslatedLabel
from .identifier import Identifier
from .psa import Psa
from .relationship_count import RelationshipCount
from .shipment_arrival import ShipmentArrival
from .shipment_departure import ShipmentDeparture
from .source_count_info import SourceCountInfo
from .status import Status


class CoreEntity(EntitySummary):
    """
    The attributes fields common to most entities.
    """

    id: str = pydantic_v1.Field()
    """
    Unique identifier of the entity
    """

    owner: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    User or group that created the entity, if applicable. Undefined for Sayari entities.
    """

    type: Entities = pydantic_v1.Field()
    """
    The [entity type](/sayari-library/ontology/entities).
    """

    label: str = pydantic_v1.Field()
    """
    Display name of the entity
    """

    names: typing.Optional[typing.List[str]] = pydantic_v1.Field(default=None)
    """
    Name variations of the entity.
    """

    company_type: typing.Optional[CompanyType] = None
    registration_date: typing.Optional[EntityRegistrationDate] = None
    latest_status: typing.Optional[Status] = None
    shipment_arrival: typing.Optional[ShipmentArrival] = None
    shipment_departure: typing.Optional[ShipmentDeparture] = None
    hs_code: typing.Optional[EntityHsCode] = None
    translated_label: typing.Optional[EntityTranslatedLabel] = None
    short_label: typing.Optional[str] = None
    identifiers: typing.List[Identifier]
    addresses: typing.Optional[typing.List[str]] = pydantic_v1.Field(default=None)
    """
    List of physical addresses associated with the entity. See more [here](/sayari-library/ontology/attributes#address)
    """

    date_of_birth: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Birth date of a person. See more [here](/sayari-library/ontology/attributes#date-of-birth)
    """

    countries: typing.List[Country] = pydantic_v1.Field()
    """
    Entity [country](/sayari-library/ontology/enumerated-types#country)
    """

    closed: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    True if the entity existed in the past but not at the present time, otherwise false. Always false for data curation.
    """

    related_entities_count: int
    user_related_entities_count: int
    relationship_counts: RelationshipCount
    user_relationship_counts: RelationshipCount
    attribute_counts: typing.Any
    user_attribute_counts: typing.Any
    trade_count: typing.Dict[str, int]
    record_count: int
    user_record_count: int
    source_counts: typing.Dict[str, SourceCountInfo] = pydantic_v1.Field()
    """
    Number of records associated with the entity, grouped by source.
    """

    psa: typing.Optional[Psa] = None
    risk: EntityRisk = pydantic_v1.Field()
    """
    [Risk factors](/sayari-library/ontology/risk-factors) associated with the entity.
    """

    created: typing.Optional[str] = None
    updated: typing.Optional[str] = None
    edited_by: typing.Optional[str] = None
    editable: typing.Optional[bool] = None
    upload: typing.Optional[str] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .notification import Notification
from .notification_additional_information import NotificationAdditionalInformation
from .notification_type import NotificationType
from .project_notification_data import ProjectNotificationData
from .project_notifications_response import ProjectNotificationsResponse
from .resource_notification_data import ResourceNotificationData
from .resource_notifications_response import ResourceNotificationsResponse

__all__ = [
    "Notification",
    "NotificationAdditionalInformation",
    "NotificationType",
    "ProjectNotificationData",
    "ProjectNotificationsResponse",
    "ResourceNotificationData",
    "ResourceNotificationsResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.query_encoder import encode_query
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.history_response import HistoryResponse
from .types.usage_response import UsageResponse


class InfoClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_usage(
        self,
        *,
        from_: typing.Optional[dt.date] = None,
        to: typing.Optional[dt.date] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UsageResponse:
        """
        The usage endpoint provides a simple interface to retrieve information on usage made by your API account. This includes both views per API path and credits consumed. The time period for the usage query is also specified in the response and whether or not this includes total usage.

        Parameters
        ----------
        from_ : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the starting time period to obtain usage stats. In the format YYYY-MM-DD

        to : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the ending time period to obtain usage stats. In the format YYYY-MM-DD

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UsageResponse
            OK

        Examples
        --------
        import datetime

        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.info.get_usage(
            from_=datetime.date.fromisoformat(
                "2023-01-15",
            ),
            to=datetime.date.fromisoformat(
                "2023-01-15",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/usage"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "from": str(from_) if from_ is not None else None,
                            "to": str(to) if to is not None else None,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(UsageResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_history(
        self,
        *,
        events: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        from_: typing.Optional[dt.date] = None,
        to: typing.Optional[dt.date] = None,
        size: typing.Optional[int] = None,
        token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HistoryResponse:
        """
        The history endpoint return a user's event history.

        Parameters
        ----------
        events : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            The type of events to filter on.

        from_ : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the starting time period for the events. In the format YYYY-MM-DD

        to : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the ending time period for the events. In the format YYYY-MM-DD

        size : typing.Optional[int]
            Size to limit number of events returned

        token : typing.Optional[str]
            Pagination token to retrieve the next page of results

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HistoryResponse
            OK

        Examples
        --------
        import datetime

        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.info.get_history(
            events="string",
            from_=datetime.date.fromisoformat(
                "2023-01-15",
            ),
            to=datetime.date.fromisoformat(
                "2023-01-15",
            ),
            size=1,
            token="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/history"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "events": events,
                            "from": str(from_) if from_ is not None else None,
                            "to": str(to) if to is not None else None,
                            "size": size,
                            "token": token,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(HistoryResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncInfoClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_usage(
        self,
        *,
        from_: typing.Optional[dt.date] = None,
        to: typing.Optional[dt.date] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UsageResponse:
        """
        The usage endpoint provides a simple interface to retrieve information on usage made by your API account. This includes both views per API path and credits consumed. The time period for the usage query is also specified in the response and whether or not this includes total usage.

        Parameters
        ----------
        from_ : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the starting time period to obtain usage stats. In the format YYYY-MM-DD

        to : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the ending time period to obtain usage stats. In the format YYYY-MM-DD

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UsageResponse
            OK

        Examples
        --------
        import datetime

        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.info.get_usage(
            from_=datetime.date.fromisoformat(
                "2023-01-15",
            ),
            to=datetime.date.fromisoformat(
                "2023-01-15",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/usage"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "from": str(from_) if from_ is not None else None,
                            "to": str(to) if to is not None else None,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(UsageResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_history(
        self,
        *,
        events: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        from_: typing.Optional[dt.date] = None,
        to: typing.Optional[dt.date] = None,
        size: typing.Optional[int] = None,
        token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HistoryResponse:
        """
        The history endpoint return a user's event history.

        Parameters
        ----------
        events : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            The type of events to filter on.

        from_ : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the starting time period for the events. In the format YYYY-MM-DD

        to : typing.Optional[dt.date]
            An ISO 8601 encoded date string indicating the ending time period for the events. In the format YYYY-MM-DD

        size : typing.Optional[int]
            Size to limit number of events returned

        token : typing.Optional[str]
            Pagination token to retrieve the next page of results

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HistoryResponse
            OK

        Examples
        --------
        import datetime

        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.info.get_history(
            events="string",
            from_=datetime.date.fromisoformat(
                "2023-01-15",
            ),
            to=datetime.date.fromisoformat(
                "2023-01-15",
            ),
            size=1,
            token="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/history"),
            params=encode_query(
                jsonable_encoder(
                    remove_none_from_dict(
                        {
                            "events": events,
                            "from": str(from_) if from_ is not None else None,
                            "to": str(to) if to is not None else None,
                            "size": size,
                            "token": token,
                            **(
                                request_options.get("additional_query_parameters", {})
                                if request_options is not None
                                else {}
                            ),
                        }
                    )
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(HistoryResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .client_name import ClientName
from .company_type import CompanyType
from .coordinate import Coordinate
from .core_entity import CoreEntity
from .embedded_entity import EmbeddedEntity
from .entity_details import EntityDetails
from .entity_hs_code import EntityHsCode
from .entity_matches import EntityMatches
from .entity_registration_date import EntityRegistrationDate
from .entity_relationships import EntityRelationships
from .entity_risk import EntityRisk
from .entity_summary import EntitySummary
from .entity_translated_label import EntityTranslatedLabel
from .identifier import Identifier
from .possibly_same_as import PossiblySameAs
from .possibly_same_as_data import PossiblySameAsData
from .possibly_same_as_match import PossiblySameAsMatch
from .psa import Psa
from .psa_entity import PsaEntity
from .psa_match_keys import PsaMatchKeys
from .record_details import RecordDetails
from .referenced_by import ReferencedBy
from .referenced_by_data import ReferencedByData
from .referenced_by_data_type import ReferencedByDataType
from .relationship_count import RelationshipCount
from .relationship_data import RelationshipData
from .relationship_info import RelationshipInfo
from .risk_data import RiskData
from .risk_level import RiskLevel
from .risk_value import RiskValue
from .search_field import SearchField
from .shipment_arrival import ShipmentArrival
from .shipment_departure import ShipmentDeparture
from .source_count_info import SourceCountInfo
from .status import Status

__all__ = [
    "ClientName",
    "CompanyType",
    "Coordinate",
    "CoreEntity",
    "EmbeddedEntity",
    "EntityDetails",
    "EntityHsCode",
    "EntityMatches",
    "EntityRegistrationDate",
    "EntityRelationships",
    "EntityRisk",
    "EntitySummary",
    "EntityTranslatedLabel",
    "Identifier",
    "PossiblySameAs",
    "PossiblySameAsData",
    "PossiblySameAsMatch",
    "Psa",
    "PsaEntity",
    "PsaMatchKeys",
    "RecordDetails",
    "ReferencedBy",
    "ReferencedByData",
    "ReferencedByDataType",
    "RelationshipCount",
    "RelationshipData",
    "RelationshipInfo",
    "RiskData",
    "RiskLevel",
    "RiskValue",
    "SearchField",
    "ShipmentArrival",
    "ShipmentDeparture",
    "SourceCountInfo",
    "Status",
]
